/*
  gethostbyname.c  --  get the IP address of a host
  Copyright (C) 1999 Steffen Solyga <solyga@absinth.net>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include	"gethostbyname.h"

int
display_help( char* pn ) {
  fprintf( HELP_CHANNEL, "%s v%s (%s): ", pn, VERSION_NUMBER, DATE_OF_LAST_MOD );
  fprintf( HELP_CHANNEL, "Get the IP address of a host.\n" );
  fprintf( HELP_CHANNEL, "Flowers & bug reports to %s.\n", MY_EMAIL_ADDRESS );
  fprintf( HELP_CHANNEL, "Usage: %s [options] host\n", pn );
  fprintf( HELP_CHANNEL, "switches:\n");
  fprintf( HELP_CHANNEL, "  -h\t help, write this info to %s and exit sucessfully\n", HELP_CHANNEL==stdout?"stdout":"stderr" );
  fprintf( HELP_CHANNEL, "  -V\t print version and compilation info to %s and exit sucessfully\n", VERSION_CHANNEL==stdout?"stdout":"stderr" );
  return( 0 );
}


int
display_version( char* pn ) {
  fprintf( VERSION_CHANNEL, "%s v%s (%s)\n", pn, VERSION_NUMBER, DATE_OF_LAST_MOD );
/*
  fprintf( VERSION_CHANNEL, "compilation settings:\n" );
*/
  return( 0 );
}


ulong
get_ipaddr_n_by_name( char* hostname ) {
/* return binary address in network byte order */
  struct hostent* ent= gethostbyname( hostname );
  if( ent == NULL ) return( -1 );
  if( ent->h_addr_list[0] == NULL ) return( -1 );
  return( *( (ulong*)ent->h_addr_list[0] ) );
}


int
main( int argc, char** argv ) {
/*
SOLYGA --------------------
SOLYGA main() gethostbyname

SOLYGA started      : Wed Oct 13 01:25:30 CEST 1999 @beast
*/
  int retval= RETVAL_OK;
  int c;
  char* hostname= NULL;
  ulong ipaddr_n;
  char str_ipaddr[]= "000.000.000.000";
  UCHAR* p= (UCHAR*)&ipaddr_n;
#if defined OS_TYPE_AIX
  extern int errno;
  extern char* sys_errlist[];
#endif
/***** process options *****/
  while( (c=getopt(argc,argv,"hV")) != EOF ) {
    switch( c ) {
      case 'h': /* display help and exit sucessfully */
        display_help(*argv);
        retval= RETVAL_OK; goto DIE_NOW;
      case 'V': /* display version to VERSION_CHANNEL and exit sucessfully */
        display_version( *argv );
        retval= RETVAL_OK; goto DIE_NOW;
      case '?': /* refer to -h and exit unsucessfully */
        fprintf( ERROR_CHANNEL, "Try `%s -h' for more information.\n", *argv );
        retval= RETVAL_ERROR; goto DIE_NOW;
      default : /* program error */
        fprintf( ERROR_CHANNEL, "%s: Options bug! E-mail me at %s.\n",
                 *argv, MY_EMAIL_ADDRESS );
        retval= RETVAL_BUG; goto DIE_NOW;
    }
  }

/* get IP address */
  if( optind<argc )
    hostname= argv[optind];
  else {
    fprintf( ERROR_CHANNEL, "%s: No hostname given.\n", *argv );
    retval=RETVAL_NO_NAME; goto DIE_NOW;
  }
  if( (ipaddr_n=get_ipaddr_n_by_name( hostname )) == -1 ) {
    fprintf( ERROR_CHANNEL, "%s: Couldn't resolve hostname `%s'. %s.\n",
             *argv, hostname, h_errlist[h_errno] );
    switch( h_errno ) {
      case HOST_NOT_FOUND:
        retval= RETVAL_HOST_NOT_FOUND;
        break;
      case TRY_AGAIN:
        retval= RETVAL_TRY_AGAIN;
        break;
      case NO_RECOVERY:
        retval= RETVAL_NO_RECOVERY;
        break;
      case NO_ADDRESS:
        retval= RETVAL_NO_ADDRESS;
        break;
      default:
        retval= RETVAL_ERROR;
    }
    goto DIE_NOW;
  }
  sprintf( str_ipaddr, "%d.%d.%d.%d",
            p[0], p[1], p[2], p[3] );
  fprintf( STD_CHANNEL, "%s\n", str_ipaddr );

DIE_NOW:
  exit( retval );
}
